/*
 * wxcitems.cc
 * wxWindows command processing: panel items
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"
#include "wx_ipc.h"

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wxextend.h"

// Button
class wxExtensionButton: public wxButton
{
 public:
  wxExtensionButton(wxPanel *panel, wxFunction func, char *label, int x = -1, int y = -1,
           int width = -1, int height = -1, long style = 0, char *name = "button"):
    wxButton(panel, func, label, x, y, width, height, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_BUTTON);
 }
  wxExtensionButton(wxPanel *panel, wxFunction func, wxBitmap *bitmap, int x = -1, int y = -1,
           int width = -1, int height = -1, long style = 0, char *name = "button"):
   wxButton(panel, func, bitmap, x, y, width, height, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_BUTTON);
 }
 ~wxExtensionButton(void)
 {
   wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_BUTTON, "OnDelete");
   if (func)
     (* (void (*)(long)) func) ((long)this);
   wxDeleteThingInTable((long)this);
 }
};

// Group box
class wxExtensionGroupBox: public wxGroupBox
{
 public:
  wxExtensionGroupBox(wxPanel *panel, char *label, int x = -1, int y = -1,
           int width = -1, int height = -1, long style = 0, char *name = "groupBox"):
    wxGroupBox(panel, label, x, y, width, height, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_GROUP_BOX);
 }
 ~wxExtensionGroupBox(void)
 {
   wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_GROUP_BOX, "OnDelete");
   if (func)
     (* (void (*)(long)) func) ((long)this);
   wxDeleteThingInTable((long)this);
 }
};

// Message
class wxExtensionMessage: public wxMessage
{
 public:
  wxExtensionMessage(wxPanel *panel, char *label, int x = -1, int y = -1, long style = 0, char *name = "message"):
    wxMessage(panel, label, x, y, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_MESSAGE);
 }
  wxExtensionMessage(wxPanel *panel, wxBitmap *bitmap, int x = -1, int y = -1, long style = 0, char *name = "message"):
    wxMessage(panel, bitmap, x, y, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_MESSAGE);
 }
 ~wxExtensionMessage(void)
 {
   wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_MESSAGE, "OnDelete");
   if (func)
     (* (void (*)(long)) func) ((long)this);
   wxDeleteThingInTable((long)this);
 }
};

// Text
class wxExtensionText: public wxText
{
 public:
  wxExtensionText(wxPanel *panel, wxFunction func, char *label, char *value = "",
         int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "text"):
    wxText(panel, func, label, value, x, y, width, height, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_TEXT);
 }

 ~wxExtensionText(void)
 {
   wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_TEXT, "OnDelete");
   if (func)
     (* (void (*)(long)) func) ((long)this);
   wxDeleteThingInTable((long)this);
 }
};

// Multi-line Text
class wxExtensionMultiText: public wxMultiText
{
 public:
  wxExtensionMultiText(wxPanel *panel, wxFunction func, char *label, char *value = "",
         int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "multiText"):
    wxMultiText(panel, func, label, value, x, y, width, height, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_MULTI_TEXT);
 }

 ~wxExtensionMultiText(void)
 {
   wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_MULTI_TEXT, "OnDelete");
   if (func)
     (* (void (*)(long)) func) ((long)this);
   wxDeleteThingInTable((long)this);
 }
};

// Check box
class wxExtensionCheckBox: public wxCheckBox
{
 public:
  wxExtensionCheckBox(wxPanel *panel, wxFunction func, char *label,
         int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "checkBox"):
    wxCheckBox(panel, func, label, x, y, width, height, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_CHECK_BOX);
 }

 ~wxExtensionCheckBox(void)
 {
   wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_CHECK_BOX, "OnDelete");
   if (func)
     (* (void (*)(long)) func) ((long)this);
   wxDeleteThingInTable((long)this);
 }
};

// Slider
class wxExtensionSlider: public wxSlider
{
 public:
  wxExtensionSlider(wxPanel *panel, wxFunction func, char *label,
         int value, int min_value, int max_value, int width, int x = -1, int y = -1, long style = 0, char *name = "slider"):
    wxSlider(panel, func, label, value, min_value, max_value, width, x, y, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_SLIDER);
 }

 ~wxExtensionSlider(void)
 {
   wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_SLIDER, "OnDelete");
   if (func)
     (* (void (*)(long)) func) ((long)this);
   wxDeleteThingInTable((long)this);
 }
};

// List box
class wxExtensionListBox: public wxListBox
{
 public:
  wxExtensionListBox(wxPanel *panel, wxFunction func, char *label,
                 Bool multiple = FALSE, int x = -1, int y = -1, int width = -1, int height = -1,
                 long style = 0, char *name = "listBox"):
    wxListBox(panel, func, label, multiple, x, y, width, height, 0, NULL, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_LIST_BOX);
 }

 ~wxExtensionListBox(void)
 {
   wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_LIST_BOX, "OnDelete");
   if (func)
     (* (void (*)(long)) func) ((long)this);
   wxDeleteThingInTable((long)this);
 }
};

// Choice
class wxExtensionChoice: public wxChoice
{
 public:
  wxExtensionChoice(wxPanel *panel, wxFunction func, char *label,
                 int x = -1, int y = -1, int width = -1, int height = -1, int n = 0, char **choices = NULL,
                 long style = 0, char *name = "choice"):
    wxChoice(panel, func, label, x, y, width, height, n, choices, style, name)
 {
   wxStoreThingInTable((long)this, this, wxTYPE_CHOICE);
 }

 ~wxExtensionChoice(void)
 {
   wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_CHOICE, "OnDelete");
   if (func)
     (* (void (*)(long)) func) ((long)this);
   wxDeleteThingInTable((long)this);
 }
};

// RadioBox
class wxExtensionRadioBox: public wxRadioBox
{
public:
    wxExtensionRadioBox(wxPanel *panel, wxFunction func, char *label,
                        int x = -1, int y = -1, int width = -1,
                        int height = -1, int n = 0, char **choices = NULL,
                        int majorDim = 0, long style = 0, char *name = "radioBox"):
    wxRadioBox(panel, func, label, x, y, width, height, n, choices,
               majorDim, style, name)
    {
        wxStoreThingInTable((long)this, this, wxTYPE_RADIO_BOX);
    }

    ~wxExtensionRadioBox(void)
    {
        wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_RADIO_BOX,
                                                           "OnDelete");
        if (func)
            (* (void (*)(long)) func) ((long)this);
        wxDeleteThingInTable((long)this);
    }
};


class wxExtensionMenuBar: public wxMenuBar
{
 public:
  wxExtensionMenuBar(void)
  {
    wxStoreThingInTable((long)this, this, wxTYPE_MENU_BAR);
  }
  ~wxExtensionMenuBar(void)
  {
     wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_MENU_BAR, "OnDelete");
     if (func)
       (* (void (*)(long)) func) ((long)this);
     wxDeleteThingInTable((long)this);
  }
};

class wxExtensionMenu: public wxMenu
{
 public:
  wxExtensionMenu(char *title, wxFunction func):wxMenu(title, func)
  {
    wxStoreThingInTable((long)this, this, wxTYPE_MENU);
  }
  ~wxExtensionMenu(void)
  {
     wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_MENU, "OnDelete");
     if (func)
       (* (void (*)(long)) func) ((long)this);
     wxDeleteThingInTable((long)this);
  }
};

#if USE_GAUGE
// Gauge
class wxExtensionGauge: public wxGauge
{
 public:
  wxExtensionGauge(wxPanel *panel, char *label, int range, int x = -1, int y = -1,
           int width = -1, int height = -1, long style = 0, char *name = "gauge"):
    wxGauge(panel, label, range, x, y, width, height, style, name)
 {
    wxStoreThingInTable((long)this, this, wxTYPE_GAUGE);
 }
 ~wxExtensionGauge(void)
 {
   wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_GAUGE, "OnDelete");
   if (func)
     (* (void (*)(long)) func) ((long)this);
   wxDeleteThingInTable((long)this);
 }
};
#endif

/************************************************************************
 * GENERIC PANEL ITEM FUNCTIONS
 *
 ************************************************************************/

// Set this button to be the default
Bool wxcPanelItemSetDefault(WXID id)
{
  wxButton *item = (wxButton *)wxGetTypedObject(id, wxTYPE_BUTTON);
  if (item)
  {
    item->SetDefault();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in panel-item-set-default: could not find item " << id << ".\n";
    return FALSE;
  }
}

// Set label
Bool wxcPanelItemSetLabel(WXID id, char *label)
{
  if (!label)
    return FALSE;

  wxItem *item = (wxItem *)wxGetTypedObject(id, wxTYPE_ITEM);
  if (item)
  {
    item->SetLabel(label);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in panel-item-set-label: could not find item " << id << ".\n";
    return FALSE;
  }
}

// Get label
char *wxcPanelItemGetLabel(WXID id)
{
  wxItem *item = (wxItem *)wxGetTypedObject(id, wxTYPE_ITEM);
  if (item)
  {
    char *label = item->GetLabel();
    return label;
  }
  else
  {
    ExtensionError << "Error in panel-item-get-label: could not find item " << id << ".\n";
    return NULL;
  }
}

/************************************************************************
 * PANEL ITEM CALLBACKS
 *
 ************************************************************************/
static wxEvent *wxCurrentEvent = NULL;

void wxExtensionButtonCallback(wxExtensionButton& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_BUTTON, "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, wxTYPE_COMMAND_EVENT);

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);

    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

void wxExtensionTextCallback(wxExtensionText& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_TEXT, "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, wxTYPE_COMMAND_EVENT);

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

void wxExtensionMultiTextCallback(wxExtensionMultiText& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_MULTI_TEXT, "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, wxTYPE_COMMAND_EVENT);

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

void wxExtensionListBoxCallback(wxExtensionListBox& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_LIST_BOX, "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, wxTYPE_COMMAND_EVENT);

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

void wxExtensionChoiceCallback(wxExtensionChoice& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_CHOICE, "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, wxTYPE_COMMAND_EVENT);

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}


void wxExtensionRadioBoxCallback(wxExtensionRadioBox& item,
                                 wxCommandEvent& event)
{
    wxExtensionFunction func =
        wxFindExtensionCallback(wxTYPE_RADIO_BOX, "OnCommand");
    wxCurrentEvent = &event;
    if (func)
    {
        wxStoreThingInTable((long)&event, &event, wxTYPE_COMMAND_EVENT);

        Bool handled = FALSE;
    
        (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item,
                                                 (long)&event);
        wxDeleteThingInTable((long)&event);
    }
    wxCurrentEvent = NULL;
}

void wxExtensionCheckBoxCallback(wxExtensionCheckBox& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_CHECK_BOX, "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, wxTYPE_COMMAND_EVENT);

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

void wxExtensionSliderCallback(wxExtensionSlider& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(wxTYPE_SLIDER, "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, wxTYPE_COMMAND_EVENT);

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

/************************************************************************
 * PANEL ITEM FUNCTIONS
 *
 ************************************************************************/

// Create a button
WXID wxcButtonCreate(WXID parentId, char *label,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionButton *button = new wxExtensionButton(panel, (wxFunction)wxExtensionButtonCallback,
     label, (int)x, (int)y, (int)width, (int)height, style, name);

    return (long)button;
  }
  else
  {
    ExtensionError << "Error in button-create: could not find parent panel " << parentId << ".\n";
    return TRUE;
  }
}

// Create a button (string version)
WXID wxcButtonCreate(WXID parentId, char *label,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcButtonCreate(parentId, label, x, y, width, height, style, name);
}

// Create a button from a bitmap
WXID wxcButtonCreateFromBitmap(WXID parentId, long bitmapId,
  int x, int y, int width, int height, long style, char *name)
{
#if (defined(wx_msw) && !defined(FAFA_LIB))
  ExtensionError << "Error in button-create-from-bitmap:\nBitmap buttons not supported in this version.\n";
  return -1;
#else
  wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(bitmapId, wxTYPE_BITMAP);

  if (!bitmap)
  {
    ExtensionError << "Error in button-create-from-bitmap: could not find bitmap " << bitmapId << ".\n";
    return -1;
  }

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionButton *button = new wxExtensionButton(panel, (wxFunction)wxExtensionButtonCallback,
     bitmap, (int)x, (int)y, (int)width, (int)height, style, name);

    return (long)button;
  }
  else
  {
    ExtensionError << "Error in button-create-from-bitmap: could not find parent panel " << parentId << ".\n";
    return -1;
  }
#endif
}

// Create a button from a bitmap (string version)
WXID wxcButtonCreateFromBitmap(WXID parentId, WXID bitmapId,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcButtonCreateFromBitmap(parentId, bitmapId, x, y, width, height,
    style, name);
}


// Change button label (string version)
Bool wxcButtonSetLabel(WXID id, char *label)
{
  wxButton *but = (wxButton *)wxGetTypedObject(id, wxTYPE_BUTTON);
  if (but)
  {
    if (label == NULL) label = "";
    but->SetLabel(label);
    return TRUE;
  }
  ExtensionError << "Error in button-set-label: could not find button " << id << ".\n";
  return FALSE;
}

// Change button label (Bitmap version)
Bool wxcButtonSetLabel(WXID id, WXID bitmapId)
{
  wxButton *but = (wxButton *)wxGetTypedObject(id, wxTYPE_BUTTON);
  if (but)
  {
    wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(bitmapId, wxTYPE_BITMAP);
    if (bitmap) { 
      but->SetLabel(bitmap);
      return TRUE;
    }
    ExtensionError << "Error in button-set-label: could not find bitmap " << bitmapId << ".\n";
  } else
    ExtensionError << "Error in button-set-label: could not find button " << id << ".\n";
  return FALSE;
}


// Create a group box
WXID wxcGroupBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionGroupBox *groupBox = new wxExtensionGroupBox(panel, 
     label, (int)x, (int)y, (int)width, (int)height, style, name);

    return (long)groupBox;
  }
  else
  {
    ExtensionError << "Error in group-box-create: could not find parent panel " << parentId << ".\n";
    return TRUE;
  }
}

// Create a group box (string version)
WXID wxcGroupBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcGroupBoxCreate(parentId, label, x, y, width, height, style, name);
}

// Create a text widget
WXID wxcTextCreate(WXID parentId, char *label, char *value,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;
  if (!value)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionText *text = new wxExtensionText(panel, (wxFunction)wxExtensionTextCallback,
                   label, value, x, y, width, height, style, name);

    return (long)text;
  }
  else
  {
    ExtensionError << "Error in text-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a text widget (string version)
WXID wxcTextCreate(WXID parentId, char *label, char *value,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcTextCreate(parentId, label, value, x, y, width, height, style, name);
}

// Set value
Bool wxcTextSetValue(WXID id, char *val)
{
  if (!val)
    return FALSE;

  wxText *item = (wxText *)wxGetTypedObject(id, wxTYPE_TEXT);
  if (item)
  {
    item->SetValue(val);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in text-set-value: could not find text item " << id << ".\n";
    return FALSE;
  }
}

// Get value
char *wxcTextGetValue(WXID id)
{
  wxText *item = (wxText *)wxGetTypedObject(id, wxTYPE_TEXT);
  if (item)
  {
    char *val = item->GetValue();
    return val;
  }
  else
  {
    ExtensionError << "Error in text-get-value: could not find text item " << id << ".\n";
    return NULL;
  }
}

// Create a multi-line text widget
WXID wxcMultiTextCreate(WXID parentId, char *label, char *value,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;
  if (!value)
    return -1;
  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionMultiText *text = new wxExtensionMultiText(panel, (wxFunction)wxExtensionMultiTextCallback, label, value,
      (int)x, (int)y, (int)width, (int)height, style, name);

    return (long)text;
  }
  else
  {
    ExtensionError << "Error in text-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a multi-line text widget (string version)
WXID wxcMultiTextCreate(WXID parentId, char *label, char *value,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcMultiTextCreate(parentId, label, value, x, y, width, height, style, name);
}

// Set value
Bool wxcMultiTextSetValue(WXID id, char *val)
{
  if (!val)
    return FALSE;

  wxMultiText *item = (wxMultiText *)wxGetTypedObject(id, wxTYPE_MULTI_TEXT);
  if (item)
  {
    item->SetValue(val);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in multi-text-set-value: could not find text item " << id << ".\n";
    return FALSE;
  }
}

// Get value
char *wxcMultiTextGetValue(WXID id)
{
  wxMultiText *item = (wxMultiText *)wxGetTypedObject(id, wxTYPE_MULTI_TEXT);
  if (item)
  {
    char *val = item->GetValue();
    return val;
  }
  else
  {
    ExtensionError << "Error in multi-text-get-value: could not find text item " << id << ".\n";
    return NULL;
  }
}

// Create a check box widget
WXID wxcCheckBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionCheckBox *box = new wxExtensionCheckBox(panel, (wxFunction)wxExtensionCheckBoxCallback, label,
      (int)(int)x, (int)y, (int)width, (int)height, style, name);

    return (long)box;
  }
  else
  {
    ExtensionError << "Error in check-box-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a check box widget (string version)
WXID wxcCheckBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcCheckBoxCreate(parentId, label, x, y, width, height, style, name);
}

// Set value
Bool wxcCheckBoxSetValue(WXID id, Bool val)
{
  wxCheckBox *item = (wxCheckBox *)wxGetTypedObject(id, wxTYPE_CHECK_BOX);
  if (item)
  {
    item->SetValue(val);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in check-box-set-value: could not find check box " << id << ".\n";
    return FALSE;
  }
}

// Get value
Bool wxcCheckBoxGetValue(WXID id)
{
  wxCheckBox *item = (wxCheckBox *)wxGetTypedObject(id, wxTYPE_CHECK_BOX);
  if (item)
  {
    Bool val = item->GetValue();
    return val;
  }
  else
  {
    ExtensionError << "Error in check-box-get-value: could not find check box " << id << ".\n";
    return FALSE;
  }
}

// Create a slider widget
WXID wxcSliderCreate(WXID parentId, char *label,
  int value, int min_value, int max_value, int width, int x, int y, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionSlider *slider = new wxExtensionSlider(panel, (wxFunction)wxExtensionSliderCallback, label,
    (int)value, (int)min_value, (int)max_value, (int)width, (int)x, (int)y, style, name);

    return (long)slider;
  }
  else
  {
    ExtensionError << "Error in slider-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a slider widget (string version)
WXID wxcSliderCreate(WXID parentId, char *label,
  int value, int min_value, int max_value, int width, int x, int y, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcSliderCreate(parentId, label, value, min_value, max_value, width, x, y, style, name);
}

// Set value
Bool wxcSliderSetValue(WXID id, int val)
{
  wxSlider *item = (wxSlider *)wxGetTypedObject(id, wxTYPE_SLIDER);
  if (item)
  {
    item->SetValue(val);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in slider-set-value: could not find slider " << id << ".\n";
    return FALSE;
  }
}

// Get value
int wxcSliderGetValue(WXID id)
{
  wxSlider *item = (wxSlider *)wxGetTypedObject(id, wxTYPE_SLIDER);
  if (item)
  {
    int val = item->GetValue();
    return val;
  }
  else
  {
    ExtensionError << "Error in slider-get-value: could not find slider " << id << ".\n";
    return -1;
  }
}

// Create a message widget
WXID wxcMessageCreate(WXID parentId, char *label, int x, int y, long style, char *name)
{
  if (!label)
    return -1;
  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionMessage *mess = new wxExtensionMessage(panel, label, (int)x, (int)y, style, name);

    return (long)mess;
  }
  else
  {
    ExtensionError << "Error in message-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a message widget (string version)
WXID wxcMessageCreate(WXID parentId, char *label, int x, int y, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcMessageCreate(parentId, label, x, y, style, name);
}

// Create a message widget
WXID wxcMessageCreateFromBitmap(WXID parentId, WXID bitmapId, int x, int y, long style, char *name)
{
#if (defined(wx_msw) && (!defined(FAFA_LIB) || !USE_BITMAP_MESSAGE))
  ExtensionError << "Error in message-create-from-bitmap:\nBitmap buttons not supported in this version.\n";
  return -1;
#else
  wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(bitmapId, wxTYPE_BITMAP);

  if (!bitmap)
  {
    ExtensionError << "Error in message-create-from-bitmap: could not find bitmap " << bitmapId << ".\n";
    return -1;
  }

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionMessage *mess = new wxExtensionMessage(panel, bitmap, x, y, style, name);

    return (long)mess;
  }
  else
  {
    ExtensionError << "Error in message-create-from-bitmap: could not find parent panel " << parentId << ".\n";
    return 0;
  }
#endif
  return 0;
}

// Create a message widget (string version)
WXID wxcMessageCreateFromBitmap(WXID parentId, WXID bitmapId, int x, int y, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcMessageCreateFromBitmap(parentId, bitmapId, x, y, style, name);
}

// Create a listbox widget
WXID wxcListBoxCreate(WXID parentId, char *label, Bool multiple,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionListBox *item = new wxExtensionListBox(panel, (wxFunction)wxExtensionListBoxCallback, label,
     (Bool)multiple, (int)x, (int)y, (int)width, (int)height, style, name);

    return (long)item;
  }
  else
  {
    ExtensionError << "Error in list-box-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a listbox widget (string version)
WXID wxcListBoxCreate(WXID parentId, char *label, Bool multiple,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcListBoxCreate(parentId, label, multiple, x, y, width, height, style, name);
}

// Append a string (and optional client data)
Bool wxcListBoxAppend(WXID id, char *item_string, char *client_data)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    item->Append(item_string, client_data);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in list-box-append: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Find a string and return ID
int wxcListBoxFindString(WXID id, char *item_string)
{
  if (!item_string)
    return -1;

  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    return item->FindString(item_string);
  }
  else
  {
    ExtensionError << "Error in list-box-find-string: could not find list box " << id << ".\n";
    return -1;
  }
}

// Clear
Bool wxcListBoxClear(WXID id)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    item->Clear();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in list-box-clear: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Set the current selection
Bool wxcListBoxSetSelection(WXID id, int sel)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    item->SetSelection(sel);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in list-box-set-selection: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Deselect an item
Bool wxcListBoxDeselect(WXID id, int sel)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    item->Deselect(sel);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in list-box-deselect: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Get the current selection
int wxcListBoxGetSelection(WXID id)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    return item->GetSelection();
  }
  else
  {
    ExtensionError << "Error in list-box-get-selection: could not find list box " << id << ".\n";
    return -1;
  }
}

// Get string selection
char *wxcListBoxGetStringSelection(WXID id)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    char *s = item->GetStringSelection();
    return s;
  }
  else
  {
    ExtensionError << "Error in list-box-get-string-selection: could not find list box " << id << ".\n";
    return NULL;
  }
}

// Set string selection
Bool wxcListBoxSetStringSelection(WXID id, char *sel)
{
  if (!sel)
    return FALSE;

  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    return item->SetStringSelection(sel);
  }
  else
  {
    ExtensionError << "Error in list-box-set-string-selection: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Get number of items
int wxcListBoxNumber(WXID id)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    return item->Number();
  }
  else
  {
    ExtensionError << "Error in list-box-number: could not find list box " << id << ".\n";
    return -1;
  }
}

// Delete one item
Bool wxcListBoxDelete(WXID id, int n)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    item->Delete(n);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in list-box-delete: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Get client data for an item
char *wxcListBoxGetClientData(WXID id, int sel)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    char *s = item->GetClientData(sel);
    return s;
  }
  else
  {
    ExtensionError << "Error in list-box-get-client-data: could not find list box " << id << ".\n";
    return NULL;
  }
}

// Get string for integer ID
char *wxcListBoxGetString(WXID id, int sel)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    char *s = item->GetString(sel);
    return s;
  }
  else
  {
    ExtensionError << "Error in list-box-get-string: could not find list box " << id << ".\n";
    return NULL;
  }
}

// Get selections
int *wxListBoxSelections = NULL;
int wxListBoxNSelections = 0;
int wxListBoxCurrentSelection;

int wxcListBoxGetFirstSelection(WXID id)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    wxListBoxNSelections = item->GetSelections(&wxListBoxSelections);
    wxListBoxCurrentSelection = 1;
    if (wxListBoxNSelections > 0)
      return wxListBoxSelections[0];
    else
      return -1;
  }
  else
  {
    ExtensionError << "Error in list-box-get-first-selection: could not find list box " << id << ".\n";
    return -1;
  }
}

int wxcListBoxGetNextSelection(void)
{
  if (wxListBoxCurrentSelection >= wxListBoxNSelections)
    return -1;

  wxListBoxCurrentSelection ++;

  return wxListBoxSelections[wxListBoxCurrentSelection - 1];
}

long wxcListBoxSetFirstItem(WXID id, long sel)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, wxTYPE_LIST_BOX);
  if (item)
  {
    item->SetFirstItem((int)sel);
    return 1;
  }
  else
  {
    ExtensionError << "Error in list-box-set-first-item: could not find list box " << id << ".\n";
    return 0;
  }
}

// Generate choice widget creator, used only indirectly by C-only or CLIPS-only
// functions. This is so we can have a generic function for all languages.
// Create a choice widget
WXID wxcChoiceCreate(WXID parentId, char *label, int x, int y,
  int width, int height, int n, char **choices, long style, char *name)
{
  if (!label)
    return -1;
  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionChoice *item = new wxExtensionChoice(panel, (wxFunction)wxExtensionChoiceCallback, label,
     (int)x, (int)y, (int)width, (int)height, n, choices, style, name);

    return (long)item;
  }
  else
  {
    ExtensionError << "Error in choice-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a choice widget (string version)
WXID wxcChoiceCreate(WXID parentId, char *label, int x, int y,
  int width, int height, int n, char **choices, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcChoiceCreate(parentId, label, x, y, width, height, n, choices, style, name);
}

// Append a string
Bool wxcChoiceAppend(WXID id, char *item_string)
{
  if (!item_string)
    return FALSE;

  wxChoice *item = (wxChoice *)wxGetTypedObject(id, wxTYPE_CHOICE);
  if (item)
  {
    item->Append(item_string);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in choice-append: could not find choice " << id << ".\n";
    return FALSE;
  }
}

// Find a string and return ID
int wxcChoiceFindString(WXID id, char *item_string)
{
  if (!item_string)
    return -1;

  wxChoice *item = (wxChoice *)wxGetTypedObject(id, wxTYPE_CHOICE);
  if (item)
  {
    return item->FindString(item_string);
  }
  else
  {
    ExtensionError << "Error in choice-find-string: could not find choice " << id << ".\n";
    return -1;
  }
}

// Clear
Bool wxcChoiceClear(WXID id)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, wxTYPE_CHOICE);
  if (item)
  {
    item->Clear();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in choice-clear: could not find choice " << id << ".\n";
    return FALSE;
  }
}

// Set the current selection
Bool wxcChoiceSetSelection(WXID id, int sel)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, wxTYPE_CHOICE);
  if (item)
  {
    item->SetSelection(sel);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in choice-set-selection: could not find choice " << id << ".\n";
    return FALSE;
  }
}

// Get the current selection
int wxcChoiceGetSelection(WXID id)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, wxTYPE_CHOICE);
  if (item)
  {
    return item->GetSelection();
  }
  else
  {
    ExtensionError << "Error in choice-get-selection: could not find choice " << id << ".\n";
    return -1;
  }
}

// Get string selection
char *wxcChoiceGetStringSelection(WXID id)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, wxTYPE_CHOICE);
  if (item)
  {
    char *s = item->GetStringSelection();
    return s;
  }
  else
  {
    ExtensionError << "Error in choice-get-string-selection: could not find choice " << id << ".\n";
    return NULL;
  }
}

// Set string selection
Bool wxcChoiceSetStringSelection(WXID id, char *sel)
{
  if (!sel)
    return FALSE;

  wxChoice *item = (wxChoice *)wxGetTypedObject(id, wxTYPE_CHOICE);
  if (item)
  {
    return item->SetStringSelection(sel);
  }
  else
  {
    ExtensionError << "Error in choice-set-string-selection: could not find choice " << id << ".\n";
    return FALSE;
  }
}

// Get string for integer ID
char *wxcChoiceGetString(WXID id, int sel)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, wxTYPE_CHOICE);
  if (item)
  {
    char *s = item->GetString(sel);
    return s;
  }
  else
  {
    ExtensionError << "Error in choice-get-string: could not find choice " << id << ".\n";
    return NULL;
  }
}

#if USE_GAUGE
// Create a gauge
WXID wxcGaugeCreate(WXID parentId, char *label,
  int range, int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
  if (panel)
  {
    wxExtensionGauge *gauge = new wxExtensionGauge(panel, label, (int)range, (int)x, (int)y,
      (int)width, (int)height, style, name);

    return (long)gauge;
  }
  else
  {
    ExtensionError << "Error in gauge-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a gauge (string version)
WXID wxcGaugeCreate(WXID parentId, char *label,
  int range, int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcGaugeCreate(parentId, label, range, x, y, width, height, style, name);
}

// Set value
Bool wxcGaugeSetValue(WXID id, int value)
{
  wxGauge *item = (wxGauge *)wxGetTypedObject(id, wxTYPE_GAUGE);
  if (item)
  {
    item->SetValue(value);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in gauge-set-value: could not find gauge item " << id << ".\n";
    return FALSE;
  }
}

// Set shadow width
Bool wxcGaugeSetShadowWidth(WXID id, int width)
{
  wxGauge *item = (wxGauge *)wxGetTypedObject(id, wxTYPE_GAUGE);
  if (item)
  {
    item->SetShadowWidth(width);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in gauge-set-shadow-width: could not find gauge item " << id << ".\n";
    return FALSE;
  }
}

// Set bezel face width
Bool wxcGaugeSetBezelFace(WXID id, int width)
{
  wxGauge *item = (wxGauge *)wxGetTypedObject(id, wxTYPE_GAUGE);
  if (item)
  {
    item->SetBezelFace(width);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in gauge-set-bezel-face: could not find gauge item " << id << ".\n";
    return FALSE;
  }
}

Bool wxcGaugeSetRange(WXID id, int value)
{
  wxGauge *item = (wxGauge *)wxGetTypedObject(id, wxTYPE_GAUGE);
  if (item)
  {
    item->SetRange(value);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in gauge-set-range: could not find gauge item " << id << ".\n";
    return FALSE;
  }
}


#endif // USE_GAUGE

// Create a menu
WXID wxcMenuCreate(char *title, wxFunction func)
{
  wxExtensionMenu *menu = new wxExtensionMenu(title, func);

  return (long)menu;
}

// Append a string or submenu
Bool wxcMenuAppend(WXID id, int item_id, char *menu_string, WXID submenu_id, char *help_string, Bool checkable)
{
  if (!menu_string)
    return FALSE;

  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, wxTYPE_MENU);
  if (!menu)
  {
    ExtensionError << "Error in menu-append: could not find menu " << id << ".\n";
    return FALSE;
  }

  if (submenu_id > -1)
  {
    wxMenu *submenu = (wxMenu *)wxGetTypedObject(submenu_id, wxTYPE_MENU);
    if (submenu)
    {
      menu->Append((int)item_id, menu_string, submenu, help_string);
      return TRUE;
    }
    else
    {
      ExtensionError << "Error in menu-append: could not find submenu " << submenu_id << "\n";
      return FALSE;
    }
  }
  else
  {
    menu->Append((int)item_id, menu_string, help_string, checkable);
    return TRUE;
  }
}


// Append a Checkable string 
Bool wxcMenuAppend(WXID id, int item_id, char *menu_string, char *help_string, Bool checkable)
{
  if (!menu_string)
    return FALSE;

  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, wxTYPE_MENU);
  if (!menu)
  {
    ExtensionError << "Error in menu-append: could not find menu " << id << ".\n";
    return FALSE;
  }

  menu->Append((int)item_id, menu_string, help_string, checkable);
  return TRUE;
}

// Append a separator
Bool wxcMenuAppendSeparator(WXID id)
{
  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, wxTYPE_MENU);
  if (menu)
  {
    menu->AppendSeparator();
    return TRUE;
  }
  ExtensionError << "Error in menu-append-separator: could not find menu " << id << ".\n";
  return FALSE;
}

// Menu Break
Bool wxcMenuBreak(WXID id)
{
  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, wxTYPE_MENU);
  if (menu)
  {
    menu->Break();
    return TRUE;
  }
  ExtensionError << "Error in menu-break: could not find menu " << id << ".\n";
  return FALSE;
}

// Enable an item
Bool wxcMenuEnable(WXID id, int item_id, Bool enabled)
{
  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, wxTYPE_MENU);
  if (menu)
  {
    menu->Enable(item_id, enabled);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in menu-enable: could not find menu " << id << ".\n";
    return FALSE;
  }
}

// Check an item
Bool wxcMenuCheck(WXID id, int item_id, Bool checked)
{
  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, wxTYPE_MENU);
  if (menu)
  {
    menu->Check(item_id, checked);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in menu-check: could not find menu " << id << ".\n";
    return FALSE;
  }
}

long wxcMenuSetLabel(WXID id, long item_id, char *label)
{
  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, wxTYPE_MENU);
  if (menu)
  {
    menu->SetLabel((int)item_id, label);
    return 1;
  }
  else
  {
    ExtensionError << "Error in menu-set-label: could not find menu "<< id << ".\n";
    return 0;
  }
}

// Create a menu bar
WXID wxcMenuBarCreate(void)
{
  wxExtensionMenuBar *menu_bar = new wxExtensionMenuBar;

  return (long)menu_bar;
}

// Append a menu
Bool wxcMenuBarAppend(WXID id, WXID menu_id, char *menu_string)
{
  if (!menu_string)
    return FALSE;
  wxMenuBar *menu_bar = (wxMenuBar *)wxGetTypedObject(id, wxTYPE_MENU_BAR);
  if (menu_bar)
  {
    wxMenu *menu = (wxMenu *)wxGetTypedObject(menu_id, wxTYPE_MENU);
    if (menu)
    {
      menu_bar->Append(menu, menu_string);
      return TRUE;
    }
    else
    {
      ExtensionError << "Error in menu-bar-append: could not find menu " << menu_id << ".\n";
      return FALSE;
    }
  }
  else
  {
    ExtensionError << "Error in menu-bar-append: could not find menu bar " << id << ".\n";
    return FALSE;
  }
}

// Enable an item
Bool wxcMenuBarEnable(WXID id, int item_id, Bool enabled)
{
  wxMenuBar *menu_bar = (wxMenuBar *)wxGetTypedObject(id, wxTYPE_MENU_BAR);
  if (menu_bar)
  {
    menu_bar->Enable(item_id, enabled);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in menu-bar-enable: could not find menu bar " << id << ".\n";
    return FALSE;
  }
}

// Check an item
Bool wxcMenuBarCheck(WXID id, int item_id, Bool checked)
{
  wxMenuBar *menu_bar = (wxMenuBar *)wxGetTypedObject(id, wxTYPE_MENU_BAR);
  if (menu_bar)
  {
    menu_bar->Check(item_id, checked);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in menu-bar-check: could not find menu bar " << id << ".\n";
    return FALSE;
  }
}

// Get Check status of an item
Bool wxcMenuBarChecked(WXID id, int item_id)
{
  wxMenuBar *menu_bar = (wxMenuBar *)wxGetTypedObject(id, wxTYPE_MENU_BAR);
  if (menu_bar)
  {
    return menu_bar->Checked(item_id);
  }
  else
  {
    ExtensionError << "Error in menu-bar-checked: could not find menu bar " << id << ".\n";
    return FALSE;
  }
}


WXID wxcRadioBoxCreate(WXID parentId, char *label, int x, int y,
                       int width, int height, int n, char **choices,
                       int majorDim, long style, char *name)
{
    if (!label)
        return -1;
    wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, wxTYPE_PANEL);
    if (panel)
    {
        wxExtensionRadioBox *item =
            new wxExtensionRadioBox(panel,
                                    (wxFunction)wxExtensionRadioBoxCallback,
                                    label, (int)x, (int)y,
                                    (int)width, (int)height,
                                    n, choices, majorDim, style, name);

        return (long)item;
    }
    else
    {
        ExtensionError << "Error in radio-box-create: could not find parent panel " << parentId << ".\n";
        return 0;
    }
}

// Create a radiobox widget (string style version)
WXID wxcRadioBoxCreate(WXID parentId, char *label, int x, int y,
  int width, int height, int n, char **choices, int majorDim, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcRadioBoxCreate(parentId, label, x, y, width, height, n, choices, majorDim, style, name);
}


long wxcRadioBoxSetSelection(WXID id, long sel)
{
  wxRadioBox *item = (wxRadioBox *)wxGetTypedObject(id, wxTYPE_RADIO_BOX);
  if (item)
  {
    item->SetSelection((int)sel);
    return 1;
  }
  else
  {
    ExtensionError << "Error in radio-box-set-selection: could not find RadioBox " << id << ".\n";
    return 0;
  }
}


long wxcRadioBoxGetSelection(WXID id)
{
  wxRadioBox *item = (wxRadioBox *)wxGetTypedObject(id, wxTYPE_RADIO_BOX);
  if (item)
  {
    return item->GetSelection();
  }
  else
  {
    ExtensionError << "Error in radio-box-get-selection: could not find radiobox " << id << ".\n";
    return -1;
  }
}
