/*
 * wxcwin.cc
 * wxWindows command processing: windows
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"
/* #include "wx_canvs.h" */

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wxextend.h"
#include "wxextenp.h"

// Record what frames and dialogs are used so we can clear
// them up if necessary
wxList wxExtensionFrameList;

/************************************************************************
 * SPECIAL-PURPOSE WINDOWS/WIDGETS
 *
 ************************************************************************/

// Dialog box
class wxExtensionDialogBox:public wxDialogBox
{
  public:
  wxExtensionDialogBox (wxWindow * parent, char *title,
			Bool modal = FALSE,
			int x = -1, int y = -1,
			int width = -1, int height = -1, long style = 0, char *name = "dialogBox"):
    wxDialogBox (parent, title, modal, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, wxTYPE_DIALOG_BOX);
    wxExtensionFrameList.Append (this);

    wxCanvasDC *dc = GetDC ();
    if (dc)
        wxStoreThingInTable ((long) dc, dc, wxTYPE_DC_CANVAS);
  }

   ~wxExtensionDialogBox ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_DIALOG_BOX, "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);

    wxDeleteThingInTable ((long) this);
    wxExtensionFrameList.DeleteObject (this);

    wxCanvasDC *dc = GetDC ();
    wxDeleteThingInTable ((long) dc);
  }
  void OnSize (int w, int h);
  Bool OnClose (void);
  void OnPaint (void);
  void OnEvent (wxMouseEvent & event);
  void OnDefaultAction (wxItem * item);
};

void wxExtensionDialogBox::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_DIALOG_BOX, "OnSize");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxDialogBox::OnSize (w, h);
    }
  else
    wxDialogBox::OnSize (w, h);
}

Bool wxExtensionDialogBox::OnClose (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_DIALOG_BOX, "OnClose");
  if (func)
    {
      Bool handled = FALSE;
      Bool returnValue = (*(Bool (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        return wxDialogBox::OnClose ();
      else
	return returnValue;
    }
  else
    return wxDialogBox::OnClose ();
}

void wxExtensionDialogBox::OnPaint (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_DIALOG_BOX, "OnPaint");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        wxDialogBox::OnPaint ();
    }
  else
    wxDialogBox::OnPaint ();
}

void wxExtensionDialogBox::OnEvent (wxMouseEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_DIALOG_BOX, "OnEvent");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, wxTYPE_MOUSE_EVENT);

      Bool handled = FALSE;

      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxDialogBox::OnEvent (event);
    }
  else
    wxDialogBox::OnEvent (event);
}

void wxExtensionDialogBox::OnDefaultAction (wxItem * item)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_DIALOG_BOX, "OnDefaultAction");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) item);

      if (!handled)
        wxDialogBox::OnDefaultAction (item);
    }
  else
    wxDialogBox::OnDefaultAction (item);
}

// Panel
class wxExtensionPanel:public wxPanel
{
  public:
  wxExtensionPanel (wxFrame * parent,
		    int x = -1, int y = -1,
		    int width = -1, int height = -1, long style = 0, char *name = "panel"):
    wxPanel (parent, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, wxTYPE_PANEL);
    wxCanvasDC *dc = GetDC ();
    if (dc)
        wxStoreThingInTable ((long) dc, dc, wxTYPE_DC_CANVAS);
  }

   ~wxExtensionPanel ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_PANEL, "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);

    wxDeleteThingInTable ((long) this);
    wxCanvasDC *dc = GetDC ();
    wxDeleteThingInTable ((long) dc);
  }

  void OnSize (int w, int h);
  void OnPaint (void);
  void OnEvent (wxMouseEvent & event);
  void OnDefaultAction (wxItem * item);
};

void wxExtensionPanel::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_PANEL, "OnSize");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxPanel::OnSize (w, h);
    }
  else
    wxPanel::OnSize (w, h);
}

void wxExtensionPanel::OnPaint (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_PANEL, "OnPaint");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        wxPanel::OnPaint ();
    }
  else
    wxPanel::OnPaint ();
}

void wxExtensionPanel::OnEvent (wxMouseEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_PANEL, "OnEvent");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, wxTYPE_MOUSE_EVENT);

      Bool handled = FALSE;

      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxPanel::OnEvent (event);
    }
  else
    wxPanel::OnEvent (event);
}

void wxExtensionPanel::OnDefaultAction (wxItem * item)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_PANEL, "OnDefaultAction");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) item);

      if (!handled)
        wxPanel::OnDefaultAction (item);
    }
  else
    wxPanel::OnDefaultAction (item);
}

// Canvas
class wxExtensionCanvas:public wxCanvas
{
  public:
  wxExtensionCanvas (wxWindow * parent,
		     int x = -1, int y = -1,
		     int width = -1, int height = -1, long style = 0, char *name = "canvas"):
    wxCanvas (parent, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, wxTYPE_CANVAS);
    wxCanvasDC *dc = GetDC ();
      wxStoreThingInTable ((long) dc, dc, wxTYPE_DC_CANVAS);
  }

   ~wxExtensionCanvas ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_CANVAS, "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);

    wxDeleteThingInTable ((long) this);
    wxCanvasDC *dc = GetDC ();
    wxDeleteThingInTable ((long) dc);
  }

  void OnSize (int w, int h);

  void OnChar (wxKeyEvent & event);
  void OnEvent (wxMouseEvent & event);
  void OnPaint (void);
};

void wxExtensionCanvas::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_CANVAS, "OnSize");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxCanvas::OnSize (w, h);
    }
  else
    wxCanvas::OnSize (w, h);
}

void wxExtensionCanvas::OnChar (wxKeyEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_CANVAS, "OnChar");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, wxTYPE_KEY_EVENT);

      Bool handled = FALSE;
      (*(void (*)(Bool *, long, long, long)) func)
	(&handled, (long) this, event.KeyCode (), (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxCanvas::OnChar (event);
    }
  else
    wxCanvas::OnChar (event);
}

void wxExtensionCanvas::OnEvent (wxMouseEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_CANVAS, "OnEvent");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, wxTYPE_MOUSE_EVENT);

      Bool handled = FALSE;

      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxCanvas::OnEvent (event);
    }
  else
    wxCanvas::OnEvent (event);
}

void wxExtensionCanvas::OnPaint (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_CANVAS, "OnPaint");
  if (func)
    {
      Bool handled = FALSE;

      (*(void (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        wxCanvas::OnPaint ();
    }
}

// Text subwindow
class wxExtensionTextWindow:public wxTextWindow
{
  public:
  wxExtensionTextWindow (wxWindow * parent,
			 int x = -1, int y = -1,
			 int width = -1, int height = -1, long style = 0, char *name = "textWindow"):
    wxTextWindow (parent, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, wxTYPE_TEXT_WINDOW);
  }

   ~wxExtensionTextWindow ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_TEXT_WINDOW, "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);
    wxDeleteThingInTable ((long) this);
  }

  void OnSize (int w, int h);
  void OnChar (wxKeyEvent & event);
};

void wxExtensionTextWindow::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_TEXT_WINDOW, "OnSize");
  if (func)
    {
      Bool handled = FALSE;

      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxTextWindow::OnSize (w, h);
    }
  else
    wxTextWindow::OnSize (w, h);
}

void wxExtensionTextWindow::OnChar (wxKeyEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_TEXT_WINDOW, "OnChar");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, wxTYPE_KEY_EVENT);

      Bool handled = FALSE;
      (*(void (*)(Bool *, long, long, long)) func)
	(&handled, (long) this, event.KeyCode (), (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxTextWindow::OnChar (event);
    }
  else
    wxTextWindow::OnChar (event);
}

// Frame
class wxExtensionFrame:public wxFrame
{
  public:
  wxExtensionFrame (wxFrame * parent, char *title,
		    int x = -1, int y = -1, int width = -1, int height = -1,
		    long style = wxSDI | wxDEFAULT_FRAME, char *name = "frame"):
     wxFrame (parent, title, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, wxTYPE_FRAME);
    wxExtensionFrameList.Append (this);
  }

   ~wxExtensionFrame ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_FRAME, "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);

    wxDeleteThingInTable ((long) this);
    wxExtensionFrameList.DeleteObject (this);
  }

  void OnSize (int w, int h);
  void OnMenuCommand (int id);
  void OnMenuSelect (int id);
  void OnActivate (Bool active);
  Bool OnClose (void);
  Bool OnCharHook (wxKeyEvent& event);
};

void wxExtensionFrame::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_FRAME, "OnSize");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxFrame::OnSize (w, h);
    }
  else
    wxFrame::OnSize (w, h);
}

void wxExtensionFrame::OnMenuCommand (int id)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_FRAME, "OnMenuCommand");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int)) func) (&handled, (long) this, id);

      if (!handled)
        wxFrame::OnMenuCommand (id);
    }
  else
    wxFrame::OnMenuCommand (id);
}

void wxExtensionFrame::OnMenuSelect (int id)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_FRAME, "OnMenuSelect");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int)) func) (&handled, (long) this, id);

      if (!handled)
        wxFrame::OnMenuSelect (id);
    }
  else
    wxFrame::OnMenuSelect (id);
}

void wxExtensionFrame::OnActivate (Bool active)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_FRAME, "OnActivate");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, Bool)) func) (&handled, (long) this, active);

      if (!handled)
        wxFrame::OnActivate (active);
    }
  else
    wxFrame::OnActivate (active);
}

Bool wxExtensionFrame::OnClose (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_FRAME, "OnClose");
  if (func)
    {
      Bool handled = FALSE;
      Bool returnValue = (*(Bool (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        return wxFrame::OnClose ();
      else
	return returnValue;
    }
  else
    return wxFrame::OnClose ();
}

Bool wxExtensionFrame::OnCharHook (wxKeyEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (wxTYPE_WINDOW, "OnCharHook");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, wxTYPE_KEY_EVENT);

      Bool handled = FALSE;
      Bool res = (*(Bool (*)(Bool *, long, long)) func)
	(&handled, (long) this, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        return wxFrame::OnCharHook (event);
      else return res;
    }
  else
    return wxFrame::OnCharHook (event);
}

/************************************************************************
 * GENERIC WINDOW FUNCTIONS
 *
 ************************************************************************/

// Delete window
Bool wxcWindowDelete (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      delete win;
      wxDeleteThingInTable (id);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-delete: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Show window
Bool wxcWindowShow (WXID id, Bool show)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->Show (show);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-show: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Associate a menu to a window as a Popup 
Bool wxcWindowPopupMenu (WXID id, WXID menuId, float x, float y)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      wxMenu *menu = (wxMenu *) wxGetTypedObject (menuId, wxTYPE_MENU);
      if (menu)
	{
	  return win->PopupMenu (menu, x, y) ? TRUE : FALSE;
	}
      ExtensionError << "\
Error in window-popup-menu: could not find " << "menu " << menuId << ".\n";
    }
  else
    ExtensionError << "\
Error in window-popup-menu: could not find " << "window " << id << ".\n";
  return FALSE;			// Error

}


// Get window parent
WXID wxcWindowGetParent (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      wxWindow *parent = win->GetParent ();
      if (parent)
	return (long) parent;
      else
	return FALSE;
    }
  else
    {
      ExtensionError << "Error in window-get-parent: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Get window width
int wxcWindowGetWidth (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      int w, h;
      win->GetSize (&w, &h);
      return w;
    }
  else
    {
      ExtensionError << "Error in window-get-width: could not find window " << id << ".\n";
      return 0;
    }
}

// Get window height
int wxcWindowGetHeight (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      int w, h;
      win->GetSize (&w, &h);
      return h;
    }
  else
    {
      ExtensionError << "Error in window-get-height: could not find window " << id << ".\n";
      return 0;
    }
}

// Get window client width
int wxcWindowGetClientWidth (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      int w, h;
      win->GetClientSize (&w, &h);
      return w;
    }
  else
    {
      ExtensionError << "Error in window-get-client-width: could not find window " << id << ".\n";
      return 0;
    }
}

// Get window client height
int wxcWindowGetClientHeight (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      int w, h;
      win->GetClientSize (&w, &h);
      return h;
    }
  else
    {
      ExtensionError << "Error in window-get-client-height: could not find window " << id << ".\n";
      return 0;
    }
}

// Get window x position
int wxcWindowGetX (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      int x, y;
      win->GetPosition (&x, &y);
      return x;
    }
  else
    {
      ExtensionError << "Error in window-get-x: could not find window " << id << ".\n";
      return 0;
    }
}

// Get window y position
int wxcWindowGetY (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      int x, y;
      win->GetPosition (&x, &y);
      return y;
    }
  else
    {
      ExtensionError << "Error in window-get-y: could not find window " << id << ".\n";
      return 0;
    }
}

// Set size
Bool wxcWindowSetSize (WXID id, int x, int y, int w, int h)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->SetSize (x, y, w, h);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-set-size: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Set client size
int wxcWindowSetClientSize (WXID id, int w, int h)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->SetClientSize (w, h);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-set-client-size: could not find window " << id << ".\n";
      return 0;
    }
}

// Set focus
Bool wxcWindowSetFocus (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->SetFocus ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-set-focus: could not find window " << id << ".\n";
      return FALSE;
    }
}

Bool wxcWindowSetCursor (WXID id, WXID cursorId)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      wxCursor *cursor = (wxCursor *) wxGetTypedObject (cursorId, wxTYPE_CURSOR);
      if (cursor)
	{
	  win->SetCursor (cursor);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in window-set-cursor: could not find cursor " << cursorId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in win-set-cursor: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Fit
Bool wxcWindowFit (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->Fit ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-fit: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Centre (string version)
Bool wxcWindowCentre (WXID id, char *stringDir)
{
  int direction = wxBOTH;
  if (stringDir)
    {
      if (strcmp (stringDir, "wxBOTH") == 0)
	direction = wxBOTH;
      else if (strcmp (stringDir, "wxHORIZONTAL") == 0)
	direction = wxHORIZONTAL;
      else if (strcmp (stringDir, "wxVERTICAL") == 0)
	direction = wxVERTICAL;
      else
	{
	  ExtensionError << "Error in window-centre: unrecognised direction " << direction << "\n.";
	  ExtensionError << "Should be wxHORIZONTAL, wxVERTICAL or wxBOTH.\n";
	  return FALSE;
	}
    }

  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->Centre (direction);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-centre: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Centre
Bool wxcWindowCentre (WXID id, int direction)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->Centre (direction);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-centre: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Make this window modal (i.e. disable all other windows)
// Experimental.
Bool wxcWindowMakeModal (WXID id, Bool modal)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->MakeModal (modal);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-modal: could not find window " << id << ".\n";
      return FALSE;
    }
}

long wxcWindowGetSize (WXID id, int *w, int *h)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->GetSize (w, h);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-get-size: could not find window "
	<< id << ".\n";
      return 0;
    }
}



long wxcWindowGetClientSize (WXID id, int *w, int *h)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->GetClientSize (w, h);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-get-client-size: could not find window "
	<< id << ".\n";
      return 0;
    }
}

long wxcWindowGetPosition (WXID id, int *x, int *y)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->GetPosition (x, y);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-get-x: could not find window "
	<< id << ".\n";
      return 0;
    }
}

float wxcWindowGetCharWidth (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      return win->GetCharWidth ();
    }
  else
    {
      ExtensionError << "Error in window-get-char-width: could not find window "
	<< id << ".\n";
      return 0.0;
    }
}

float wxcWindowGetCharHeight (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      return win->GetCharHeight ();
    }
  else
    {
      ExtensionError << "Error in window-get-char-height: could not find window "
	<< id << ".\n";
      return 0.0;
    }
}

long wxcWindowGetTextExtent (WXID id, char *text, float *width, float *height)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->GetTextExtent (text, width, height);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-get-text-extent: could not find window "
	<< id << ".\n";
      return 0;
    }
}

char *wxcWindowGetName(WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      return win->GetName();
    }
  else
    {
      ExtensionError << "Error in window-get-name: could not find window "
	<< id << ".\n";
      return NULL;
    }
}

long wxcWindowEnable (WXID id, Bool enable)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, wxTYPE_WINDOW);
  if (win)
    {
      win->Enable (enable);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-enable: could not find window " << id << ".\n";
      return 0;
    }
}

/************************************************************************
 * DIALOG AND PANEL CREATION
 *
 ************************************************************************/

// Create a new dialog box
WXID wxcDialogBoxCreate (WXID parentId, char *title, Bool modal,
		    int x, int y, int width, int height, long style, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, wxTYPE_WINDOW);
      if (!parent)
	{
	  ExtensionError << "Error in dialog-box-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionDialogBox *dialog_box = new wxExtensionDialogBox (parent, title, modal,
						x, y, width, height, style, name);
  return (long) dialog_box;
}

// Create a new dialog box (string version)
WXID wxcDialogBoxCreate (WXID parentId, char *title, Bool modal,
		    int x, int y, int width, int height, char *styleString, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, wxTYPE_WINDOW);
      if (!parent)
	{
	  ExtensionError << "Error in dialog-box-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  long style = wxExtensionParseBitList (styleString);

  wxExtensionDialogBox *dialog_box = new wxExtensionDialogBox (parent, title, modal,
						x, y, width, height, style, name);
  return (long) dialog_box;
}

// Create a new panel
long wxcPanelCreate (long parentId, int x, int y, int width, int height, long style, char *name)
{
  wxFrame *frame = NULL;

  if (parentId != 0)
    {
      frame = (wxFrame *) wxGetTypedObject (parentId, wxTYPE_FRAME);
      if (!frame)
	{
	  ExtensionError << "Error in panel-create: could not find parent frame " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionPanel *panel = new wxExtensionPanel (frame, (int) x, (int) y, (int) width, (int) height, style, name);
  return (long) panel;
}

// Panel creation (string version)
long wxcPanelCreate (long parentId, int x, int y, int width, int height, char *styleString, char *name)
{
  wxFrame *frame = NULL;

  if (parentId != 0)
    {
      frame = (wxFrame *) wxGetTypedObject (parentId, wxTYPE_FRAME);
      if (!frame)
	{
	  ExtensionError << "Error in panel-create: could not find parent frame " << parentId << ".\n";
	  return 0;
	}
    }
  long style = wxExtensionParseBitList (styleString);

  wxExtensionPanel *panel = new wxExtensionPanel (frame, (int) x, (int) y, (int) width, (int) height, style, name);
  return (long) panel;
}

// Set the label position
Bool wxcPanelSetLabelPosition (WXID id, int position)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      panel->SetLabelPosition (position);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in panel-set-label-position: could not find panel " << id << ".\n";
      return TRUE;
    }
}

// Set the label position (string version)
Bool wxcPanelSetLabelPosition (WXID id, char *position)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      int pos;
      if (strcmp (position, "wxVERTICAL") == 0)
	pos = wxVERTICAL;
      else if (strcmp (position, "wxHORIZONTAL") == 0)
	pos = wxHORIZONTAL;
      else
	{
	  ExtensionError << "Error in panel-set-label-position: argument should be wxVERTICAL or wxHORIZONTAL.\n";
	  return 0;
	}
      panel->SetLabelPosition (pos);
      return 1;
    }
  else
    {
      ExtensionError << "Error in panel-set-label-position: could not find panel " << id << ".\n";
      return 0;
    }
}

// Set the panel label font
Bool wxcPanelSetLabelFont (WXID id, WXID fontId)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      wxFont *font = (wxFont *) wxGetTypedObject (fontId, wxTYPE_FONT);
      if (font)
	{
	  panel->SetLabelFont (font);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in panel-set-label-font: could not find font " << fontId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in panel-set-label-font: could not find panel " << id << ".\n";
      return FALSE;
    }
}

// Set the panel button font
Bool wxcPanelSetButtonFont (WXID id, WXID fontId)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      wxFont *font = (wxFont *) wxGetTypedObject (fontId, wxTYPE_FONT);
      if (font)
	{
	  panel->SetButtonFont (font);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in panel-set-button-font: could not find font " << fontId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in panel-set-button-font: could not find panel " << id << ".\n";
      return FALSE;
    }
}

// New line
Bool wxcPanelNewLine (WXID id)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      panel->NewLine ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in panel-new-line: not find panel " << id << ".\n";
      return FALSE;
    }
}

// Default panel or dialog box OnPaint
Bool wxcPanelOnPaint (WXID id)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      panel->wxPanel::OnPaint ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in panel-on-paint: not find panel " << id << ".\n";
      return FALSE;
    }
}

// Default panel or dialog box OnEvent
Bool wxcPanelOnEvent (WXID id, WXID eventId)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      wxMouseEvent *event = (wxMouseEvent *) wxGetTypedObject (eventId, wxTYPE_MOUSE_EVENT);
      if (event)
	{
	  panel->wxPanel::OnEvent (*event);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in panel-on-event: not find mouse event " << eventId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in panel-on-event: not find panel " << id << ".\n";
      return FALSE;
    }
}

long wxcPanelGetHorizontalSpacing (WXID id)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      return panel->GetHorizontalSpacing ();
    }
  else
    {
      ExtensionError << "Error in panel-get-horizontal-spacing: could not find panel " << id << ".\n";
      return FALSE;
    }
}


long wxcPanelGetVerticalSpacing (WXID id)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      return panel->GetVerticalSpacing ();
    }
  else
    {
      ExtensionError << "Error in panel-get-vertical-spacing: could not find panel " << id << ".\n";
      return FALSE;
    }
}

Bool wxcPanelSetHorizontalSpacing (WXID id, long spacing)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      panel->SetHorizontalSpacing ((int)spacing);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in panel-set-horizontal-spacing: could not find panel " << id << ".\n";
      return FALSE;
    }
}

Bool wxcPanelSetVerticalSpacing (WXID id, long spacing)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, wxTYPE_PANEL);
  if (panel)
    {
      panel->SetVerticalSpacing ((int)spacing);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in panel-set-vertical-spacing: could not find panel " << id << ".\n";
      return FALSE;
    }
}

// Create a new canvas
WXID wxcCanvasCreate (long parentId, int x, int y, int width, int height, long style, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, wxTYPE_WINDOW);
      if (!parent)
	{
	  ExtensionError << "Error in canvas-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionCanvas *canvas = new wxExtensionCanvas (parent, x, y, width, height, style, name);
  return (long) canvas;
}

// Create a new canvas (string version)
WXID wxcCanvasCreate (long parentId, int x, int y, int width, int height, char *styleString, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, wxTYPE_WINDOW);
      if (!parent)
	{
	  ExtensionError << "Error in canvas-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  long style = wxExtensionParseBitList (styleString);

  wxExtensionCanvas *canvas = new wxExtensionCanvas (parent, x, y, width, height, style, name);
  return (long) canvas;
}

// Set the canvas scrollbars
Bool wxcCanvasSetScrollbars (WXID id, int horiz, int vert,
			int x_length, int y_length, int x_page, int y_page,
			int x_pos, int y_pos)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, wxTYPE_CANVAS);
  if (canvas)
    {
      canvas->SetScrollbars (horiz, vert,
			     x_length, y_length,
			     x_page, y_page,
			     x_pos, y_pos);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in canvas-set-scrollbars: could not find canvas " << id << ".\n";
      return FALSE;
    }
}

// Set the canvas scrollbars
Bool wxcCanvasScroll (WXID id, int x_pos, int y_pos)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, wxTYPE_CANVAS);
  if (canvas)
    {
      canvas->Scroll (x_pos, y_pos);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in canvas-scroll: could not find canvas " << id << ".\n";
      return FALSE;
    }
}

// Get canvas's device context
WXID wxcCanvasGetDC (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, wxTYPE_CANVAS);
  if (canvas)
    {
      return (long) canvas->GetDC ();
    }
  else
    {
      ExtensionError << "Error in canvas-get-dc: could not find canvas " << id << ".\n";
      return 0;
    }
}

// Default canvas OnEvent
Bool wxcCanvasOnEvent (WXID id, WXID eventId)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, wxTYPE_CANVAS);
  if (canvas)
    {
      wxMouseEvent *event = (wxMouseEvent *) wxGetTypedObject (eventId, wxTYPE_MOUSE_EVENT);
      if (event)
	{
	  canvas->wxCanvas::OnEvent (*event);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in canvas-on-event: not find mouse event " << eventId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in canvas-on-event: not find panel " << id << ".\n";
      return FALSE;
    }
}

// Default canvas OnChar
Bool wxcCanvasOnChar (WXID id, WXID eventId)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, wxTYPE_CANVAS);
  if (canvas)
    {
      wxKeyEvent *event = (wxKeyEvent *) wxGetTypedObject (eventId, wxTYPE_KEY_EVENT);
      if (event)
	{
	  canvas->wxCanvas::OnChar (*event);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in canvas-on-char: not find key event " << eventId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in canvas-on-char: not find panel " << id << ".\n";
      return FALSE;
    }
}

// Create a new text window
WXID wxcTextWindowCreate (WXID parentId, int x, int y, int width, int height,
		     long style, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, wxTYPE_WINDOW);
      if (!parent)
	{
	  ExtensionError << "Error in text-window-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  wxExtensionTextWindow *text_window =
  new wxExtensionTextWindow (parent, x, y, width, height, style, name);
  return (long) text_window;
}

// Create a new text window (string version)
WXID wxcTextWindowCreate (long parentId, int x, int y, int width, int height,
		     char *styleString, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, wxTYPE_WINDOW);
      if (!parent)
	{
	  ExtensionError << "Error in text-window-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  long style = wxExtensionParseBitList (styleString);

  wxExtensionTextWindow *text_window =
  new wxExtensionTextWindow (parent, x, y, width, height, style, name);
  return (long) text_window;
}

// Load a file
Bool wxcTextWindowLoadFile (WXID id, char *filename)
{
  if (!filename)
    return FALSE;

  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      return win->LoadFile (filename);
    }
  else
    {
      ExtensionError << "Error in text-window-load-file: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Save a file
Bool wxcTextWindowSaveFile (WXID id, char *filename)
{
  if (!filename)
    return FALSE;

  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      return win->SaveFile (filename);
    }
  else
    {
      ExtensionError << "Error in text-window-save-file: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Write text to the window
Bool wxcTextWindowWrite (WXID id, long val)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    *win << val;
  else
    {
      ExtensionError << "Error in text-window-write: could not find text window " << id << ".\n";
      return FALSE;
    }
  return TRUE;
}

Bool wxcTextWindowWrite (WXID id, char *val)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    *win << val;
  else
    {
      ExtensionError << "Error in text-window-write: could not find text window " << id << ".\n";
      return FALSE;
    }
  return TRUE;
}

Bool wxcTextWindowWrite (WXID id, double val)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    *win << val;
  else
    {
      ExtensionError << "Error in text-window-write: could not find text window " << id << ".\n";
      return FALSE;
    }
  return TRUE;
}

// Return TRUE if modified
Bool wxcTextWindowModified (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      return win->Modified ();
    }
  else
    {
      ExtensionError << "Error in text-window-modified: could not find text window " << id << ".\n";
      return FALSE;
    }
}

Bool wxcTextWindowClear (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->Clear ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in text-window-clear: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Return TRUE if modified
Bool wxcTextWindowDiscardEdits (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->DiscardEdits ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in text-window-discard-edits: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Copy selection to clipboard
Bool wxcTextWindowCopy (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->Copy ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in text-window-copy: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Paste selection from clipboard
Bool wxcTextWindowPaste (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->Paste ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in text-window-paste: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Copy selection to clipboard, and delete.
Bool wxcTextWindowCut (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->Cut ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in text-window-cut: could not find text window " << id << ".\n";
      return FALSE;
    }
}

char *wxcTextWindowGetContents (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      return win->GetContents ();
    }
  else
    {
      ExtensionError << "Error in text-window-get-contents: could not find window "
	<< id << ".\n";
      return 0;
    }
}

Bool wxcTextWindowReplace (WXID id, long start, long end, char *text)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->Replace (start, end, text);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-replace: could not find window " << id << ".\n";
  return FALSE;
}

Bool wxcTextWindowRemove (WXID id, long start, long end)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->Remove (start, end);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-remove: could not find window " << id << ".\n";
  return FALSE;
}


Bool wxcTextWindowSetEditable (WXID id, Bool editable)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->SetEditable (editable);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-set-editable: could not find window " << id << ".\n";
  return FALSE;
}


Bool wxcTextWindowSetFont (WXID id, WXID fontId)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      wxFont *font = (wxFont *) wxGetTypedObject (fontId, wxTYPE_FONT);
      if (font)
	{
	  win->SetFont (font);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "\
Error in text-window-set-font: could not find font " << fontId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "\
Error in text-window-set-font: could not find dc " << id << ".\n";
      return FALSE;
    }
}

Bool wxcTextWindowSetSelection (WXID id, long start, long end)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->SetSelection (start, end);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-set-selection: could not find window " << id << ".\n";
  return FALSE;
}

long wxcTextWindowGetLastPosition (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    return win->GetLastPosition ();
  ExtensionError << "\
Error in text-window-get-last-position: could not find window " << id << ".\n";
  return -1;
}

Bool wxcTextWindowShowPosition (WXID id, long position)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->ShowPosition (position);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-show-position: could not find window " << id << ".\n";
  return FALSE;
}


long wxcTextWindowXYToPosition (WXID id, long x, long y)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    return win->XYToPosition (x, y);
  ExtensionError << "\
Error in text-window-XY-to-position: could not find window " << id << ".\n";
  return -1;
}

Bool wxcTextWindowPositionToXY (WXID id, long position, long *x, long *y)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (win)
    {
      win->PositionToXY (position, x, y);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-position-to-XY: could not find window " << id << ".\n";
  return FALSE;
}


// Default text window OnChar
Bool wxcTextWindowOnChar (WXID id, WXID eventId)
{
  wxTextWindow *textw = (wxTextWindow *) wxGetTypedObject (id, wxTYPE_TEXT_WINDOW);
  if (textw)
    {
      wxKeyEvent *event = (wxKeyEvent *) wxGetTypedObject (eventId, wxTYPE_KEY_EVENT);
      if (event)
	{
	  textw->wxTextWindow::OnChar (*event);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in text-window-on-char: not find key event " << eventId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in text-window-on-char: not find panel " << id << ".\n";
      return FALSE;
    }
}

/************************************************************************
 * FRAME FUNCTIONS
 *
 ************************************************************************/

// Create a new frame
WXID wxcFrameCreate (WXID parentId, char *title, int x, int y,
		int width, int height, long style, char *name)
{
  wxFrame *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxFrame *) wxGetTypedObject (parentId, wxTYPE_FRAME);
      if (!parent)
	{
	  ExtensionError << "Error in frame-create: could not find parent frame " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionFrame *new_frame = new wxExtensionFrame (parent, title, x, y,
						      width, height, style, name);
  return (long) new_frame;
}

// Create a new frame (string version)
WXID wxcFrameCreate (WXID parentId, char *title, int x, int y,
		int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList (styleString);
  wxFrame *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxFrame *) wxGetTypedObject (parentId, wxTYPE_FRAME);
      if (!parent)
	{
	  ExtensionError << "Error in frame-create: could not find parent frame " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionFrame *new_frame = new wxExtensionFrame (parent, title, x, y,
						      width, height, style, name);
  return (long) new_frame;
}

// Set the menu bar
Bool wxcFrameSetMenuBar (WXID id, WXID menubar_id)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, wxTYPE_FRAME);
  if (frame)
    {
      wxMenuBar *menu_bar = (wxMenuBar *) wxGetTypedObject (menubar_id, wxTYPE_MENU_BAR);
      if (menu_bar)
	{
	  frame->SetMenuBar (menu_bar);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in frame-set-menu-bar: could not find menu bar " << menubar_id << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in frame-set-menu-bar: could not find frame " << id << ".\n";
      return FALSE;
    }
}

// Set the toolbar for MDI operation
Bool wxcFrameSetToolBar (WXID id, WXID toolbar_id)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, wxTYPE_FRAME);
  if (frame)
    {
      wxWindow *toolbar = (wxWindow *) wxGetTypedObject (toolbar_id, wxTYPE_WINDOW);
      if (toolbar)
	{
	  frame->SetToolBar (toolbar);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in frame-set-tool-bar: could not find toolbar " << toolbar_id << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in frame-set-tool-bar: could not find frame " << id << ".\n";
      return FALSE;
    }
}

// Create the frame's status line
Bool wxcFrameCreateStatusLine (WXID id, int number)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, wxTYPE_FRAME);
  if (frame)
    {
      frame->CreateStatusLine (number);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in frame-create-status-line: could not find frame " << id << ".\n";
      return FALSE;
    }
}

// Set the status line text
Bool wxcFrameSetStatusText (WXID id, char *text, int i)
{
  if (!text)
    return FALSE;

  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, wxTYPE_FRAME);
  if (frame)
    {
      frame->SetStatusText (text, i);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in frame-set-status-text: could not find frame " << id << ".\n";
      return FALSE;
    }
}

// Set the icon
Bool wxcFrameSetIcon (WXID id, WXID iconId)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, wxTYPE_FRAME);
  if (frame)
    {
      wxIcon *icon = (wxIcon *) wxGetTypedObject (iconId, wxTYPE_ICON);
      if (icon)
	{
	  frame->SetIcon (icon);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in frame-set-icon: could not find icon " << iconId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in frame-set-icon: could not find frame " << id << ".\n";
      return FALSE;
    }
}

// Iconimize/restore the frame
Bool wxcFrameIconize (WXID id, Bool iconize)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, wxTYPE_FRAME);
  if (frame)
    {
      frame->Iconize (iconize);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in frame-iconize: could not find frame " << id << ".\n";
      return FALSE;
    }
}


Bool wxcFrameSetTitle (WXID id, char *text)
{
  if (!text)
    return FALSE;

  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, wxTYPE_FRAME);
  if (frame)
    {
      frame->SetTitle (text);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in frame-set-status-text: could not find frame "
	<< id << ".\n";
      return FALSE;
    }
}

char *wxcFrameGetTitle (WXID id)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, wxTYPE_FRAME);
  if (frame)
    {
      return frame->GetTitle ();
    }
  else
    {
      ExtensionError << "Error in frame-set-status-text: could not find frame "
	<< id << ".\n";
      return 0;
    }
}

void wxcFrameOnSize(WXID id, int w, int h)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, wxTYPE_FRAME);
  if (frame)
    {
      frame->wxFrame::OnSize(w, h);
    }
  else
    {
      ExtensionError << "Error in frame-on-size: could not find frame "
	<< id << ".\n";
    }
}
